import os

import trafaret as t


class ExistingDirectory(t.String):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def check_value(self, value):
        if not os.path.isdir(value):
            raise t.DataError(error='{} is not directory'.format(value))



from aio_yamlconfig.config import AppYAMLLoader, BaseYAMLConfig, AppYAMLConfig


async def setup_empty(app, **kwargs):
    """
    Setup empty config for manual loading configs.
    After setup user need to do:

     app.config.load("config1.yaml")
     app.config.load("config2.yaml")
     app.config.validate()
    """
    app['config'] = AppYAMLConfig(**kwargs)
    # setup attribute to access config via
    #
    # app.config['section']['name']
    app.config = app['config']


async def setup(app, config_files, base_dir, **kwargs):
    await setup_empty(app, base_dir=base_dir, **kwargs)

    for f in config_files:
        app.config.load(f)
    app.config.validate()



import collections
import os

import yaml


class AppYAMLLoader(yaml.Loader):
    """
    Empty subclass of loader to add app-specific tags.
    """


class BaseYAMLConfig(collections.Mapping):
    def __init__(self, conf_file=None, trafaret_validator=None):
        super().__init__()
        self._raw = {}
        self._validated = {}

        self._trafaret = trafaret_validator
        self._YAMLLoader = AppYAMLLoader

        if conf_file:
            self.load(conf_file)

    def load(self, conf_file):
        with open(conf_file) as cfd:
            conf = yaml.load(cfd.read(), Loader=self._YAMLLoader)
        self._raw.update(conf)

    def validate(self):
        if self._trafaret:
            self._validated = self._trafaret.check(self._raw)
        else:
            self._validated = self._raw

    def __len__(self):
        return len(self._validated)

    def __iter__(self):
        return iter(self._validated)

    def __getitem__(self, key):
        return self._validated[key]

    def __str__(self):
        if self._validated:
            return str(self._validated)
        else:
            return "<empty config>"


class AppYAMLConfig(BaseYAMLConfig):
    def __init__(self, conf_file=None, trafaret_validator=None, base_dir=None):
        super().__init__(conf_file, trafaret_validator)
        self._base_dir = base_dir
        self._setup_loader()

    def _tag_base_dir(self, loader, node):
        assert self._base_dir is not None, "configuration contains !BaseDir options, but base_dir is not defined"
        return os.path.join(self._base_dir, loader.construct_scalar(node))

    def _setup_loader(self):
        yaml.add_constructor("!BaseDir", self._tag_base_dir, Loader=self._YAMLLoader)



import os

from setuptools import setup, find_packages


classifiers = [
    'License :: OSI Approved :: BSD License',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Environment :: Web Environment',
    'Development Status :: 4 - Beta',
]


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(name='aio_yamlconfig',
      version='0.1.1',
      description='YAML configuration parser with validation.',
      long_description=read('README.rst'),
      classifiers=classifiers,
      platforms=['POSIX'],
      author='Roman Rader',
      author_email='antigluk@gmail.com',
      url='https://github.com/rrader/aio_yamlconfig',
      license='BSD',
      packages=find_packages(),
      install_requires=['trafaret', 'PyYAML'],
      include_package_data = True)
